/* test program for saving screenshots: different acceleration functions graphed in phase space */

#include <stdlib.h>
#include <GL/glut.h>
#include <math.h>
#include <iostream.h>

/* GLOBAL VARIABLES */
int wide = 500, high = 500;								/* length & width of window */
int winX = 100, winY = 100;								/* x & y positions for window */
int afunc = 1;											/* which acceleration function to use:
															0: a(x) = 0
															1: a(x) = 1
															2: a(x) = -x
															3: a(x) = -x(x^2-1) */
const int COLORS = 6;									/* number of colors */
const float X1=-3.0, X2=3.0, Y1=-3.0, Y2=3.0;			/* size of window */
const int width = (int)((X2-X1)/0.25);
const int height = (int)((Y2-Y1)/0.25);
const int length = width*height;

double points[2][length][COLORS];						/* [x-v positions][starting points][past points] */

int current=0;											/* place holder for current element in points */

double RGB[3][COLORS]= {{1,    1,    1,    .996, 1,    1   },
						{.235, .204, .447, .537, .627, 0   },
						{.702, .824, .863, .882, .910, .588}};
const double TSTEP = .05;								/* timestep */


/****************************************************************************************************/
/****************************************************************************************************/


/* FUNCTIONS */
double accel(double x)									/* acceleration function a(x) */
{
  switch(afunc)
  {
    case 0:	return 0; break;							/* no acceleration */
	case 1: return 1; break;							/* constant acceleration */
	case 2: return -1 * x; break;						/* acceleration = -x */
	case 3: return -1 * x * ((x * x) - 1);				/* weird acceleration function */
  }
  return -1;
}

double energy(double x, double v)						/* energy function E(x,v) */
{
  switch(afunc)
  {
	case 1:	return (0.5 * v * v) - x; break;			/* constant acceleration */
	case 2: return (v * v) + (x * x); break;			/* acceleration = -x */
	case 3: return (2*v*v) + (((x*x)-1)*((x*x)-1));		/* weird acceleration function */
  }
  return -1;
}

void DrawPoints()										/* plot points in phase space */
{
  for(int a = 0; a < COLORS; a++)
  {
    glColor3f(RGB[0][a], RGB[1][a], RGB[2][a]);
	for(int b = 0; b < length; b++)
	{
      glBegin( GL_POINTS );
        glVertex2f(points[0][b][(current+a)%COLORS], points[1][b][(current+a)%COLORS]);
      glEnd();
	}
  }
}

void LeapFrog()											/* calculates next points */
{
  for(int c = 0; c < length; c++)
  {
    points[0][c][(current+1)%COLORS] = points[0][c][current] + (points[1][c][current] * TSTEP);
    points[1][c][(current+1)%COLORS] = points[1][c][current] + (accel(points[0][c][current]) * TSTEP);
  }
  current = (current + 1) % COLORS;
}

void CalculatePoints()
{
  LeapFrog();
  DrawPoints();
}

void display()
{  
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);	/* clear all pixels */

  glColor3f(0.961, 0.145, 0.702);						/* magenta */
  glBegin(GL_LINES);									/* draw the axes */
    glVertex3f(X1, 0.0, 0.0);                
    glVertex3f(X2, 0.0, 0.0);                
    glVertex3f(0.0, Y1,0.0);                
    glVertex3f(0.0, Y2, 0.0);                
  glEnd();

  CalculatePoints();									/* calculates and plots points on graph */

  glFlush();
  SDL_GL_SwapBuffers();
}

void myinit()
{ 
  glClearColor(0, 0, 0, 0);								/* black */
  glMatrixMode(GL_PROJECTION);				
  glLoadIdentity();				
  glOrtho(X1, X2, Y1, Y2, -1.0, 1.0);

  int temp = 0;
  for(int d = 0; d < width; d++)						/* set initial points */
	for(int e = 0; e < height; e++)
	{
	  points[0][temp][current] = (d*0.25) + X1;
      points[1][temp][current] = (e*0.25) + Y1;
	  temp++;
	}

  for(int f = 0; f < length; f++)						/* shove velocity to 1/2 a time step off */
      points[1][f][current] = points[1][f][current] + (0.5*accel(points[0][f][current])*TSTEP);
}


/* Use the SDL program's int main() */
/*
int main(int argc, char* argv[])
{  
  glutInit(&argc, argv);								// glut's initialization routine 
  glutInitDisplayMode(GLUT_DOUBLE|GLUT_RGB);			// double buffer, RGB color 
  glutInitWindowSize(wide, high);  						// initialize window size 
  glutInitWindowPosition(winX,winY); 					// position window on screen 
  glutCreateWindow("Phase Space");						// create window with title 
  myinit();												// call initialization routine 
	
  glutDisplayFunc(display);								// call display routine 
  glutMainLoop();									 
  return 0;
}
*/